function [out1,out2,out3]=traj_pwcder(t,c,tdata,task,pRange,hRange)
% Manipulations on a 1-dimensional piecewise constant scheduling trajectory  

% Concept:
% The perod h of a scheduling trajectory p(t) is divided into N sections.  
% Over each section the time derivative of the traj. is a-priori fixed 
% and stored in tdata. So length(tdata)==N. 
% The lengths of each section are the tunable parameters of the trajectory 
% and are collected in c(1:end-1). Thus the length of the period is
% h=sum(c(1:end-1)). c(end) contains the offset of the trajectory. 
%
% p(t) is computed as follows: 
%       pdot(t)=tdata(i) if sum(c(1:i-1))<tau<sum(c(1:i)), tau=t-k*h,
%               k*h<t<(k+1)*h
%       p(t)   =c(end)+integrate(pdot(alpha),alpha=0..tau)
%
% Allowed function calls:
%   h=traj_pwcder([],c,tdata,-1); 
%       gives back the period h
%   [p,pdot]=traj_pwcder(t,c,tdata) or [p,pdot]=traj_pwcder(t,c,tdata,1)
%       evaluates the trajectory at time instants t (t can be an array) and
%       returns p(t(k)) and pdot(t(k)) k=1..length(t)
%   [A,b,Aeq]=traj_pwcder([],[],tdata,pRange,hRange)
%       Forms linear inequality and equality constraints for c s.t. if some
%       c=c1 satisfies A*c1<=b and Aeg*c1=0 then the trajectory defined by
%       (c1,tdata) satisfies the range bounds given in pRange and h bounds
%       defined in hRange, i.e. 
%       pRange(1) <= p(t) <= pRange(2), hRange(1) <= h <= hRange(2)


if nargin==0
    close all
    slopes=[2 0 -2 0 2 0];
    pRange=[0 pi];
    hRange=[0.1 10];
    c=[1 2 4 1 1 2 10]
    [Alp,Blp,Aeq]=traj_pwcder([],[],slopes,1,pRange,hRange)
    t=0:0.001:12; 
    [p,pdot]=traj_pwcder(t,c,slopes);
    subplot(2,1,1); plot(t,p)
    subplot(2,1,2); plot(t,pdot)
    return
end


if nargin==3, task=0; end;


switch task,
    case -1,
        % Give back the period
        h=sum(c(1:end-1));
        out1=h;
    case 0,
        % Evaluate the trajectory at time t
        c=c(:)';
        csc=cumsum(c(1:end-1)); h=csc(end);
        slopes=tdata; 
        const=cumsum([c(end) slopes.*c(1:end-1)]);
        Nt=numel(t); p=zeros(1,Nt); pdot=p;
        t1=mod(t,h); 
        for i=1:Nt
            k=find(t1(i)<csc,1);
            if k>1, t2=t1(i)-csc(k-1); else t2=t1(i); end;
            p(i)=const(k)+slopes(k)*t2;
            pdot(i)=slopes(k);        
        end;
        out1=p; out2=pdot; out3=[]; 
    case 1,
        % Form linear constraints to specify the allowable parameter trajectory
        slopes=tdata; 
        n=length(slopes)+1;
        M=zeros(n); M(:,end)=1; k=1;
        for i=1:length(slopes)
            if slopes(i)~=0,
                k=k+1; M(k,1:i)=slopes(1:i);
            end;      
        end;
        M(k+1:end,:)=[]; 
        A1=[M;-M]; b1=[ones(k,1)*pRange(2); -ones(k,1)*pRange(1)];
        E=[eye(n-1) zeros(n-1,1)]; 
        A2=-E; b2=zeros(n-1,1);
        I=[ones(1,n-1) 0]; 
        A3=[I;-I]; b3=[hRange(2);-hRange(1)];
        A=[A1;A2;A3]; b=[b1;b2;b3];
        Aeq=[M(end,1:end-1) 0]; beq=0;       
        out1=A; out2=b; out3=Aeq; 
    
end;
        
        

